@@ -189,12 +189,12 @@ func (m *Model) StartFolder(folder string) {
 func (m *Model) startFolderLocked(folder string) config.FolderType {
 	cfg, ok := m.folderCfgs[folder]
 	if !ok {
-		panic("cannot start nonexistent folder " + folder)
+		panic("cannot start nonexistent folder " + cfg.Description())
 	}
 
 	_, ok = m.folderRunners[folder]
 	if ok {
-		panic("cannot start already running folder " + folder)
+		panic("cannot start already running folder " + cfg.Description())
 	}
 
 	folderFactory, ok := folderFactories[cfg.Type]
@@ -777,7 +777,7 @@ func (m *Model) ClusterConfig(deviceID protocol.DeviceID, cm protocol.ClusterCon
 				"folderLabel": folder.Label,
 				"device":      deviceID.String(),
 			})
-			l.Infof("Unexpected folder ID %q sent from device %q; ensure that the folder exists and that this device is selected under \"Share With\" in the folder configuration.", folder.ID, deviceID)
+			l.Infof("Unexpected folder %s sent from device %q; ensure that the folder exists and that this device is selected under \"Share With\" in the folder configuration.", folder.Description(), deviceID)
 			continue
 		}
 		if !folder.DisableTempIndexes {
@@ -806,19 +806,19 @@ func (m *Model) ClusterConfig(deviceID protocol.DeviceID, cm protocol.ClusterCon
 						// the IndexID, or something else weird has
 						// happened. We send a full index to reset the
 						// situation.
-						l.Infof("Device %v folder %q is delta index compatible, but seems out of sync with reality", deviceID, folder.ID)
+						l.Infof("Device %v folder %s is delta index compatible, but seems out of sync with reality", deviceID, folder.Description())
 						startSequence = 0
 						continue
 					}
 
-					l.Debugf("Device %v folder %q is delta index compatible (mlv=%d)", deviceID, folder.ID, dev.MaxSequence)
+					l.Debugf("Device %v folder %s is delta index compatible (mlv=%d)", deviceID, folder.Description(), dev.MaxSequence)
 					startSequence = dev.MaxSequence
 				} else if dev.IndexID != 0 {
 					// They say they've seen an index ID from us, but it's
 					// not the right one. Either they are confused or we
 					// must have reset our database since last talking to
 					// them. We'll start with a full index transfer.
-					l.Infof("Device %v folder %q has mismatching index ID for us (%v != %v)", deviceID, folder.ID, dev.IndexID, myIndexID)
+					l.Infof("Device %v folder %s has mismatching index ID for us (%v != %v)", deviceID, folder.Description(), dev.IndexID, myIndexID)
 					startSequence = 0
 				}
 			} else if dev.ID == deviceID && dev.IndexID != 0 {
@@ -840,7 +840,7 @@ func (m *Model) ClusterConfig(deviceID protocol.DeviceID, cm protocol.ClusterCon
 					// will probably send us a full index. We drop any
 					// information we have and remember this new index ID
 					// instead.
-					l.Infof("Device %v folder %q has a new index ID (%v)", deviceID, folder.ID, dev.IndexID)
+					l.Infof("Device %v folder %s has a new index ID (%v)", deviceID, folder.Description(), dev.IndexID)
 					fs.Replace(deviceID, nil)
 					fs.SetIndexID(deviceID, dev.IndexID)
 				} else {
@@ -857,7 +857,6 @@ func (m *Model) ClusterConfig(deviceID protocol.DeviceID, cm protocol.ClusterCon
 
 		go sendIndexes(conn, folder.ID, fs, m.folderIgnores[folder.ID], startSequence, dbLocation)
 	}
-	m.fmut.Unlock()
 
 	// This breaks if we send multiple CM messages during the same connection.
 	if len(tempIndexFolders) > 0 {
@@ -883,6 +882,7 @@ func (m *Model) ClusterConfig(deviceID protocol.DeviceID, cm protocol.ClusterCon
 			changed = true
 		}
 	}
+	m.fmut.Unlock()
 
 	if changed {
 		if err := m.cfg.Save(); err != nil {
@@ -949,9 +949,10 @@ func (m *Model) handleDeintroductions(introducerCfg config.DeviceConfiguration,
 		for i := 0; i < len(folderCfg.Devices); i++ {
 			if folderCfg.Devices[i].IntroducedBy == introducerCfg.DeviceID {
 				if !foldersDevices.has(folderCfg.Devices[i].DeviceID, folderCfg.ID) {
-					// We could not find that folder shared on the introducer with the device that was introduced to us.
+					// We could not find that folder shared on the
+					// introducer with the device that was introduced to us.
 					// We should follow and unshare aswell.
-					l.Infof("Unsharing folder %q with %v as introducer %v no longer shares the folder with that device", folderCfg.ID, folderCfg.Devices[i].DeviceID, folderCfg.Devices[i].IntroducedBy)
+					l.Infof("Unsharing folder %s with %v as introducer %v no longer shares the folder with that device", folderCfg.Description(), folderCfg.Devices[i].DeviceID, folderCfg.Devices[i].IntroducedBy)
 					folderCfg.Devices = append(folderCfg.Devices[:i], folderCfg.Devices[i+1:]...)
 					i--
 					folderChanged = true
@@ -968,36 +969,26 @@ func (m *Model) handleDeintroductions(introducerCfg config.DeviceConfiguration,
 		}
 	}
 
-	// Check if we should remove some devices, if the introducer no longer shares any folder with them.
-	// Yet do not remove if we share other folders that haven't been introduced by the introducer.
-	raw := m.cfg.RawCopy()
-	deviceChanged := false
-	for i := 0; i < len(raw.Devices); i++ {
-		if raw.Devices[i].IntroducedBy == introducerCfg.DeviceID {
-			if !foldersDevices.hasDevice(raw.Devices[i].DeviceID) {
-				if foldersIntroducedByOthers.hasDevice(raw.Devices[i].DeviceID) {
-					l.Infof("Would have removed %v as %v no longer shares any folders, yet there are other folders that are shared with this device that haven't been introduced by this introducer.", raw.Devices[i].DeviceID, raw.Devices[i].IntroducedBy)
+	// Check if we should remove some devices, if the introducer no longer
+	// shares any folder with them. Yet do not remove if we share other
+	// folders that haven't been introduced by the introducer.
+	for _, device := range m.cfg.Devices() {
+		if device.IntroducedBy == introducerCfg.DeviceID {
+			if !foldersDevices.hasDevice(device.DeviceID) {
+				if foldersIntroducedByOthers.hasDevice(device.DeviceID) {
+					l.Infof("Would have removed %v as %v no longer shares any folders, yet there are other folders that are shared with this device that haven't been introduced by this introducer.", device.DeviceID, device.IntroducedBy)
 					continue
 				}
-				// The introducer no longer shares any folder with the device, remove the device.
-				l.Infof("Removing device %v as introducer %v no longer shares any folders with that device", raw.Devices[i].DeviceID, raw.Devices[i].IntroducedBy)
-				raw.Devices = append(raw.Devices[:i], raw.Devices[i+1:]...)
-				i--
-				deviceChanged = true
+				// The introducer no longer shares any folder with the
+				// device, remove the device.
+				l.Infof("Removing device %v as introducer %v no longer shares any folders with that device", device.DeviceID, device.IntroducedBy)
+				m.cfg.RemoveDevice(device.DeviceID)
+				changed = true
 			}
 		}
 	}
 
-	// We've removed a device, replace the config.
-	if deviceChanged {
-		if err := m.cfg.Replace(raw); err != nil {
-			l.Warnln("Failed to save config", err)
-		}
-		changed = true
-	}
-
 	return changed
-
 }
 
 func (m *Model) introduceDevice(device protocol.Device, introducerCfg config.DeviceConfiguration) {
@@ -1029,7 +1020,7 @@ func (m *Model) introduceDevice(device protocol.Device, introducerCfg config.Dev
 }
 
 func (m *Model) introduceDeviceToFolder(device protocol.Device, folder protocol.Folder, introducerCfg config.DeviceConfiguration) {
-	l.Infof("Sharing folder %q with %v (vouched for by introducer %v)", folder.ID, device.ID, introducerCfg.DeviceID)
+	l.Infof("Sharing folder %s with %v (vouched for by introducer %v)", folder.Description(), device.ID, introducerCfg.DeviceID)
 
 	m.deviceFolders[device.ID] = append(m.deviceFolders[device.ID], folder.ID)
 	m.folderDevices.set(device.ID, folder.ID)
@@ -1257,7 +1248,7 @@ func (m *Model) SetIgnores(folder string, content []string) error {
 
 	path := filepath.Join(cfg.Path(), ".stignore")
 
-	fd, err := osutil.CreateAtomic(path, 0644)
+	fd, err := osutil.CreateAtomic(path)
 	if err != nil {
 		l.Warnln("Saving .stignore:", err)
 		return err
@@ -1744,14 +1735,14 @@ func (m *Model) internalScanFolderSubdirs(folder string, subDirs []string) error
 
 	if err := m.CheckFolderHealth(folder); err != nil {
 		runner.setError(err)
-		l.Infof("Stopping folder %s due to error: %s", folder, err)
+		l.Infof("Stopping folder %s due to error: %s", folderCfg.Description(), err)
 		return err
 	}
 
 	if err := ignores.Load(filepath.Join(folderCfg.Path(), ".stignore")); err != nil && !os.IsNotExist(err) {
 		err = fmt.Errorf("loading ignores: %v", err)
 		runner.setError(err)
-		l.Infof("Stopping folder %s due to error: %s", folder, err)
+		l.Infof("Stopping folder %s due to error: %s", folderCfg.Description(), err)
 		return err
 	}
 
@@ -1808,7 +1799,7 @@ func (m *Model) internalScanFolderSubdirs(folder string, subDirs []string) error
 	for f := range fchan {
 		if len(batch) == batchSizeFiles || blocksHandled > batchSizeBlocks {
 			if err := m.CheckFolderHealth(folder); err != nil {
-				l.Infof("Stopping folder %s mid-scan due to folder error: %s", folder, err)
+				l.Infof("Stopping folder %s mid-scan due to folder error: %s", folderCfg.Description(), err)
 				return err
 			}
 			m.updateLocalsFromScanning(folder, batch)
@@ -1820,7 +1811,7 @@ func (m *Model) internalScanFolderSubdirs(folder string, subDirs []string) error
 	}
 
 	if err := m.CheckFolderHealth(folder); err != nil {
-		l.Infof("Stopping folder %s mid-scan due to folder error: %s", folder, err)
+		l.Infof("Stopping folder %s mid-scan due to folder error: %s", folderCfg.Description(), err)
 		return err
 	} else if len(batch) > 0 {
 		m.updateLocalsFromScanning(folder, batch)
@@ -1896,13 +1887,13 @@ func (m *Model) internalScanFolderSubdirs(folder string, subDirs []string) error
 		})
 
 		if iterError != nil {
-			l.Infof("Stopping folder %s mid-scan due to folder error: %s", folder, iterError)
+			l.Infof("Stopping folder %s mid-scan due to folder error: %s", folderCfg.Description(), iterError)
 			return iterError
 		}
 	}
 
 	if err := m.CheckFolderHealth(folder); err != nil {
-		l.Infof("Stopping folder %s mid-scan due to folder error: %s", folder, err)
+		l.Infof("Stopping folder %s mid-scan due to folder error: %s", folderCfg.Description(), err)
 		return err
 	} else if len(batch) > 0 {
 		m.updateLocalsFromScanning(folder, batch)
@@ -1958,7 +1949,12 @@ func (m *Model) generateClusterConfig(device protocol.DeviceID) protocol.Cluster
 	var message protocol.ClusterConfig
 
 	m.fmut.RLock()
-	for _, folder := range m.deviceFolders[device] {
+	// The list of folders in the message is sorted, so we always get the
+	// same order.
+	folders := m.deviceFolders[device]
+	sort.Strings(folders)
+
+	for _, folder := range folders {
 		folderCfg := m.cfg.Folders()[folder]
 		fs := m.folderFiles[folder]
 
@@ -1971,12 +1967,8 @@ func (m *Model) generateClusterConfig(device protocol.DeviceID) protocol.Cluster
 			DisableTempIndexes: folderCfg.DisableTempIndexes,
 		}
 
-		for device := range m.folderDevices[folder] {
-			// DeviceID is a value type, but with an underlying array. Copy it
-			// so we don't grab aliases to the same array later on in device[:]
-			device := device
-			// TODO: Set read only bit when relevant, and when we have per device
-			// access controls.
+		// Devices are sorted, so we always get the same order.
+		for _, device := range m.folderDevices.sortedDevices(folder) {
 			deviceCfg := m.cfg.Devices()[device]
 
 			var indexID protocol.IndexID
@@ -2299,9 +2291,9 @@ func (m *Model) runnerExchangeError(folder config.FolderConfiguration, err error
 
 	if err != nil {
 		if oldErr != nil && oldErr.Error() != err.Error() {
-			l.Infof("Folder %q error changed: %q -> %q", folder.ID, oldErr, err)
+			l.Infof("Folder %s error changed: %q -> %q", folder.Description(), oldErr, err)
 		} else if oldErr == nil {
-			l.Warnf("Stopping folder %q - %v", folder.ID, err)
+			l.Warnf("Stopping folder %s - %v", folder.Description(), err)
 		}
 		if runnerExists {
 			runner.setError(err)
@@ -2604,3 +2596,13 @@ func (s folderDeviceSet) hasDevice(dev protocol.DeviceID) bool {
 	}
 	return false
 }
+
+// sortedDevices returns the list of devices for a given folder, sorted
+func (s folderDeviceSet) sortedDevices(folder string) []protocol.DeviceID {
+	devs := make([]protocol.DeviceID, 0, len(s[folder]))
+	for dev := range s[folder] {
+		devs = append(devs, dev)
+	}
+	sort.Sort(protocol.DeviceIDs(devs))
+	return devs
+}
