@@ -445,13 +445,13 @@ func TestClusterConfig(t *testing.T) {
 		t.Errorf("Incorrect number of devices %d != 2", l)
 	}
 	if id := r.Devices[0].ID; id != device1 {
-		t.Errorf("Incorrect device ID %x != %x", id, device1)
+		t.Errorf("Incorrect device ID %s != %s", id, device1)
 	}
 	if !r.Devices[0].Introducer {
 		t.Error("Device1 should be flagged as Introducer")
 	}
 	if id := r.Devices[1].ID; id != device2 {
-		t.Errorf("Incorrect device ID %x != %x", id, device2)
+		t.Errorf("Incorrect device ID %s != %s", id, device2)
 	}
 	if r.Devices[1].Introducer {
 		t.Error("Device2 should not be flagged as Introducer")
@@ -465,13 +465,13 @@ func TestClusterConfig(t *testing.T) {
 		t.Errorf("Incorrect number of devices %d != 2", l)
 	}
 	if id := r.Devices[0].ID; id != device1 {
-		t.Errorf("Incorrect device ID %x != %x", id, device1)
+		t.Errorf("Incorrect device ID %s != %s", id, device1)
 	}
 	if !r.Devices[0].Introducer {
 		t.Error("Device1 should be flagged as Introducer")
 	}
 	if id := r.Devices[1].ID; id != device2 {
-		t.Errorf("Incorrect device ID %x != %x", id, device2)
+		t.Errorf("Incorrect device ID %s != %s", id, device2)
 	}
 	if r.Devices[1].Introducer {
 		t.Error("Device2 should not be flagged as Introducer")
@@ -921,7 +921,7 @@ func TestIgnores(t *testing.T) {
 		t.Errorf("Incorrect ignores: %v != %v", ignores, expected)
 	}
 
-	ignores, _, err = m.GetIgnores("doesnotexist")
+	_, _, err = m.GetIgnores("doesnotexist")
 	if err == nil {
 		t.Error("No error")
 	}
@@ -1773,6 +1773,8 @@ func TestScanNoDatabaseWrite(t *testing.T) {
 	}
 	defer m.SetIgnores("default", curIgn)
 	m.SetIgnores("default", nil)
+	fakeTime := time.Now().Add(5 * time.Second)
+	os.Chtimes("testdata/.stignore", fakeTime, fakeTime)
 
 	// Scan the folder twice. The second scan should be a no-op database wise
 
@@ -1789,6 +1791,8 @@ func TestScanNoDatabaseWrite(t *testing.T) {
 	// Ignore a file we know exists. It'll be updated in the database.
 
 	m.SetIgnores("default", []string{"foo"})
+	fakeTime = time.Now().Add(10 * time.Second)
+	os.Chtimes("testdata/.stignore", fakeTime, fakeTime)
 
 	m.ScanFolder("default")
 	c2 := db.Committed()
