@@ -12,6 +12,7 @@ import (
 	"strings"
 	"time"
 
+	"github.com/sasha-s/go-deadlock"
 	"github.com/syncthing/syncthing/lib/logger"
 )
 
@@ -22,7 +23,8 @@ var (
 	// We make an exception in this package and have an actual "if debug { ...
 	// }" variable, as it may be rather performance critical and does
 	// nonstandard things (from a debug logging PoV).
-	debug = strings.Contains(os.Getenv("STTRACE"), "sync") || os.Getenv("STTRACE") == "all"
+	debug       = strings.Contains(os.Getenv("STTRACE"), "sync") || os.Getenv("STTRACE") == "all"
+	useDeadlock = os.Getenv("STDEADLOCK") != ""
 )
 
 func init() {
@@ -31,5 +33,8 @@ func init() {
 	if n, err := strconv.Atoi(os.Getenv("STLOCKTHRESHOLD")); err == nil {
 		threshold = time.Duration(n) * time.Millisecond
 	}
+	if n, err := strconv.Atoi(os.Getenv("STDEADLOCK")); err == nil {
+		deadlock.Opts.DeadlockTimeout = time.Duration(n) * time.Second
+	}
 	l.Debugf("Enabling lock logging at %v threshold", threshold)
 }
